package fr.eisti.ing1.java.teaching.data;
/** 
 *	class Pool
 *	A pool of EistiStudents
 *
 *	Can be initialized from a file of Student objects (from	org.example.teaching.data)
 *  3 iterations are available :
 *		- internal construct order
 *		- ascending order (from EistiStudent order)
 *		- descending order (from EistiStudent order)
 *  Default iteration is ascending order
 *
 *	@author Matthias Colin
 *	@version 1.0 (6/6/2010)
 */

import org.example.teaching.data.Student;
import java.util.*;
import java.io.*;

public class Pool implements Iterable<EistiStudent> {

	/**
	 * name of the pool
	 */
	private String name;

	/**
	 * set of the students of the pool
	 * Set collection is the natural choice to avoid duplicate elements
	 */
	private Set<EistiStudent> students;

	/**
	 * Pool constructor
	 * initialization with a name and an empty set
 	 * @param name initial name of the pool
	 */
	public Pool(String name) {
		this.name = name;
		students = new TreeSet<EistiStudent>(); // or other Set implementation
	}

	/**
	 * Pool constructor
	 * initialization witha name and students read from a file
	 * of Student objects (from org.example.teaching.data package)
	 * Multiple instances of the same student will be remove thanks to
	 * the internal representation of the pool and the equals operator of
	 * EistiStudent class.
	 *
 	 * @param name initial name of the pool
	 * @param studentsFileName filename of the file with Student objects
	 * @throws LoadStudentsFileException if file doesn't exist, can't be openned
	 * or doesn't contain Student objects
	 */
	public Pool(String name, String studentsFileName) 
			throws LoadStudentsFileException {
		this(name);
		try {
			// open students file
			FileInputStream f = new FileInputStream(studentsFileName);
			ObjectInputStream in = new ObjectInputStream(f);
			// read students while java objects are available in file
			while (f.available() > 0) {
				// read one Student object
				Student student = (Student) in.readObject();
				// convert into EistiStudent
				EistiStudent eistiStudent = new EistiStudent(student);
				// add in the Set if not already in it
				// add operation in Set class checks if the parameter is
				// in the set according to equals operation
				students.add(eistiStudent);
			}
		} catch (Exception e) {
			// if something bad happened while loading file, throw
			// a unique exception with its cause
			throw new LoadStudentsFileException(e);
		}
	}

	/**
	 * add an EistiStudent to the pool if not in yet
	 * @param student the student to add
	 * @return true if the student has been added
	 */
	public boolean addStudent(EistiStudent student) {
		// add operation of Set does all the job
		return students.add(student);
	}

	/**
	 * remove an EistiStudent from the pool
	 * @param student the student to remove
	 * @return true if the student was in the pool
	 */
	public boolean removeStudent(EistiStudent student) {
		// remove operation of Set does all the job
		return students.remove(student);
	}

	/**
	 * size of the pool
	 * @return the number of EistiStudent in the pool
	 */
	public int size() {
		return students.size();
	}

	/**
	 * name of the pool
	 * @return the name of the pool
	 */
	public String getName() {
		return this.name;
	}

	/**
	 * iterator for the pool
	 * @param order precise the iteration order
	 * @return an new iterator
	 */
	public Iterator<EistiStudent> iterator(OrderType order) {
		switch (order) {
			case NONE:
				// return natural iterator of the Set
				return students.iterator();
			case ASCENDING:
				// construct an ordered list to iterate the students
				List<EistiStudent> l = new LinkedList<EistiStudent>(students);
				Collections.sort(l);
				return l.iterator();
			case DESCENDING:
				// construct a reversed ordered list to iterate the students
				List<EistiStudent> l2 = new LinkedList<EistiStudent>(students);
				Collections.sort(l2,Collections.reverseOrder());
				return l2.iterator();
		}
		// never happens
		return null;
	}

	/**
	 * default iterator for the pool
	 * @return an new iterator with no ordering
	 */
	@Override
	public Iterator<EistiStudent> iterator() {
		return iterator(OrderType.NONE);
	}

	/**
	 * String representation of the pool (name and size)
	 * @return the string representation
	 */
	@Override
	public String toString() {
		return name + " (" + students.size() + " students) ";
	}

}
